" PLUGIN: wiki.vim
if $WIKIROOT ==? ''
  let g:wiki_root = expand('~/documents/notes')
else
  let g:wiki_root = $WIKIROOT
endif
" filetypes to automatically enable the plugin for, seems to take file endings
" rather than vim ft
let g:wiki_filetypes = ['md', 'mkd', 'markdown', 'wiki']
let g:wiki_mappings_use_defaults = 'global'
let g:wiki_list_todos = ['[ ]', '[x]']

" wiki automatic link creation
" add .md to the end of links by default
let g:wiki_link_extension = '.md'
" add a link in the format [descr](target) by default
" not in wiki format [[link|descr]]
let g:wiki_link_target_type = 'md'

" change file opening scheme of external wiki links
function! WikiFileOpen(...) abort dict
  " use xdg-open for now -- can still customize if need be, see
  " :help wiki-config  -> g:wiki_file_open for an example
  silent execute '!xdg-open' fnameescape(self.path) '&'
  return 1
endfunction
let g:wiki_file_open = 'WikiFileOpen'

" Zettelkasten functionality

" Appends Zettelkasten timestamp to beginning of links
function! ZettelLinkCreate(text) abort
  " lowercase and replace spaces with dashes (-) to make links uniform
  let l:text=substitute(tolower(a:text), '\s\+', '-', 'g')
  " prepend datestamp
  let l:text=substitute(l:text, '^', s:Datestamp() . ' '. '\0', 'g')
  return l:text
endfunction
" sets wiki.vim to automatically apply datestamp function to created links
let g:wiki_map_link_create = 'ZettelLinkCreate'

let g:zettel_anchor_separator = ' '
let g:zettel_anchor_pattern = '[:/]\d\{10,14}'
" Returns the full path to the Zettel owning the anchor id passed in
function! s:GetZettelPath(anchor) abort
  " match first 10-digit-beggining file ending with .md
  "  e.g. 1906061330 My example-Zettel.md
  let l:tomatch='/' . trim(a:anchor) . g:zettel_anchor_separator . '.*\.md'
  let l:allnotes=split(globpath(g:wiki_root, '**'), '\n')

  " get absolute path
  let l:matched = matchstr(l:allnotes, l:tomatch)
  " make path relative to root to pass back to wiki.vim
  let l:curfpath = expand('%:p:h')
  let l:matched = substitute(l:matched, l:curfpath, '', '')

  return l:matched
endfunction
command! -nargs=1 ZettelOpen execute ":e " . s:GetZettelPath(<q-args>)

" Uses the Zettel Anchor ID instead of the whole link structure to traverse
" the whole Wiki directory from its root and opens first fitting Zettel
function! ZettelOpenAtCursor(...) abort
  let l:link = wiki#link#get()
  try
    let l:zettel = s:GetZettelPath( s:ExtractZettelAnchor(l:link.url) )

    " fall back to normal link opening otherwise
    if l:zettel ==? ''
      call wiki#link#open()
    else
      call wiki#page#open(l:zettel)
    endif
  catch /E716:/
    call wiki#link#toggle(l:link)
  catch /E37:/
    echoerr 'E37: Can''t open link before you''ve saved the current buffer.'
  endtry
endfunction
command! ZettelOpenAtCursor :call ZettelOpenAtCursor()

" Returns only the anchor id in a file name
" - returns empty string if nothing was found
function! s:ExtractZettelAnchor(file) abort
  return trim(matchstr(a:file, g:zettel_anchor_pattern), ' \t\r:/')
endfunction

" Creates a Zettelkasten compatible unique datestamp
function s:Datestamp()
  if !exists('*strftime')
    :echoerr 'Date Stamp creation needs strftime available on system!'
  endif
  return strftime('%y%m%d%H%M')
endfunction
command! Datestamp :echom s:Datestamp()
